
#include <stdio.h>
#include <windows.h>
#include "secmem.h"
#include "physmem.h"
#include "tests-private.h"
#include "tests-mapped.h"
#include "tests-secured.h"
#include "tests-physical.h"



BOOL EnablePrivilege(PWCHAR PrivilegeName)
{
  BOOL Ret = FALSE;
  HANDLE hToken; 
  LUID Luid;
  TOKEN_PRIVILEGES Privileges;

  Ret = OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken);
  if (Ret) {
    Ret = LookupPrivilegeValueW(NULL, PrivilegeName, &Luid);
    if (Ret) {
      Privileges.PrivilegeCount = 1;
      Privileges.Privileges[0].Luid = Luid;
      Privileges.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
      Ret = AdjustTokenPrivileges(hToken, FALSE, &Privileges, sizeof(TOKEN_PRIVILEGES), NULL, NULL);
    }

    CloseHandle(hToken);
  }

  return Ret;
}



int main(int argc, char **argv)
{
  BOOL Ret = FALSE;

  printf("PID of this process: %d\n", GetCurrentProcessId());
  Ret = EnablePrivilege(L"SeLockMemoryPrivilege");
  if (!Ret)
    printf("WARNING: EnablePrivilege(SeLockMemoryPrivilege): %u\n", GetLastError());

  Ret = SecMemModuleInit();
  if (Ret) {
    Ret = PhysMemModuleInit();
    if (Ret) {
      printf("PRIVATE MEMORY TESTS\n");
      SingleSmallAllocTest();
      SingleSmallAllocNoCacheTest();
      SingleSmallAllocWriteCombinedTest();
	  SingleSmallAllocGuardTest();
      SingleLargeAllocTest();
      SingleReserveTest();
      SingleWriteWatchTest();
      TwoPhaseSmallAllocTest();
      SingleProtectChangeTest();

      printf("MAPPED FILE TESTS\n");
      PagingFileSectionAllocTest();
      PagingFileSectionReserveTest();
      PagingFileSectionAllocLargeTest();

      printf("SECURE MEMORY TESTS\n");
      WholeSecureTest();
      WholeSecureTest2();
      PartialSecureTest();
      MultipleSecureTest();

      printf("PHYSICAL MEMORY TESTS\n");
	  PhysicalMemoryTest();
	  SingleAWEReserveTest();
	  SingleAWEAllocTest();

      printf("All test performed. Check with vad.exe application\n");
      getchar();

      PhysMemModuleFinit();
    } else printf("ERROR: Failed to load physmem.dll. PHYSMEM.SYS driver is probably not working properly, or is not loaded at all.\n");

    SecMemModuleFinit();
  } else printf("ERROR: Failed to load secmem.dll. SECMEM.SYS driver is probably not working properly, or is not loaded at all.\n");


  return 0;
}
