
#include <stdio.h>
#include "tests.h"
#include "tests-mapped.h"



static DWORD SectionProtect [] = {
  PAGE_EXECUTE_READ,
  PAGE_EXECUTE_READWRITE,
  PAGE_EXECUTE_WRITECOPY,
  PAGE_READONLY,
  PAGE_READWRITE,
  PAGE_WRITECOPY
};

static DWORD FileMapRights [] = {
  FILE_MAP_EXECUTE | FILE_MAP_READ,
  FILE_MAP_WRITE | FILE_MAP_READ | FILE_MAP_EXECUTE,
  FILE_MAP_READ | FILE_MAP_EXECUTE | FILE_MAP_COPY,
  FILE_MAP_READ,
  FILE_MAP_READ | FILE_MAP_WRITE,
  FILE_MAP_READ | FILE_MAP_COPY
};

static PCHAR SectionProtectStr [] = {
  "PAGE_EXECUTE_READ",
  "PAGE_EXECUTE_READWRITE",
  "PAGE_EXECUTE_WRITECOPY",
  "PAGE_READONLY",
  "PAGE_READWRITE",
  "PAGE_WRITECOPY"
};



VOID PagingFileSectionAllocTest(VOID)
{
  LONG i = 0;
  PTEST_SET TestSet = NULL;
  HANDLE SectionHandle = NULL;
  PVOID BaseAddress = NULL;

  printf("Mapping test: Mapping paging-fie backed sections (small pages)\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    for (i = 0; i < sizeof(SectionProtect) / sizeof(DWORD); ++i) {
      SectionHandle = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, SectionProtect[i], 0, 16384, NULL);
      if (SectionHandle != NULL) {
        BaseAddress = MapViewOfFile(SectionHandle, FileMapRights[i], 0, 0, 0);
        if (BaseAddress != NULL) {
          if (CreatePagingFileSectionTestAction(TestSet, SectionHandle, BaseAddress)) {
            printf("Paging-file backed section mapped %s protection at address 0x%p\n", SectionProtectStr[i], BaseAddress);
          } else {
            printf("ERROR: Failed to create test action record\n");
            UnmapViewOfFile(BaseAddress);
            CloseHandle(SectionHandle);
          }
        } else {
          printf("ERROR: Failed to map the section\n");
          CloseHandle(SectionHandle);
        }
      } else printf("ERROR: Failed to create paging file backed section with %s protection\n", SectionProtectStr[i]);
    }

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Test initialization failed: %u\n", GetLastError());

  return;
}


VOID PagingFileSectionReserveTest(VOID)
{
  LONG i = 0;
  PTEST_SET TestSet = NULL;
  HANDLE SectionHandle = NULL;
  PVOID BaseAddress = NULL;

  printf("Mapping test: Reserving paging-fie backed sections (small pages)\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    for (i = 0; i < sizeof(SectionProtect) / sizeof(DWORD); ++i) {
      SectionHandle = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, SectionProtect[i] | SEC_RESERVE, 0, 16384, NULL);
      if (SectionHandle != NULL) {
        BaseAddress = MapViewOfFile(SectionHandle, FileMapRights[i], 0, 0, 0);
        if (BaseAddress != NULL) {
          if (CreatePagingFileSectionTestAction(TestSet, SectionHandle, BaseAddress)) {
            printf("Paging-file backed section reserved with %s protection at address 0x%p\n", SectionProtectStr[i], BaseAddress);
          } else {
            printf("ERROR: Failed to create test action record\n");
            UnmapViewOfFile(BaseAddress);
            CloseHandle(SectionHandle);
          }
        } else {
          printf("ERROR: Failed to map the section\n");
          CloseHandle(SectionHandle);
        }
      } else printf("ERROR: Failed to create paging file backed section with %s protection\n", SectionProtectStr[i]);
    }

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Test initialization failed: %u\n", GetLastError());

  return;
}


VOID PagingFileSectionAllocLargeTest(VOID)
{
  LONG i = 0;
  PTEST_SET TestSet = NULL;
  HANDLE SectionHandle = NULL;
  PVOID BaseAddress = NULL;

  printf("Mapping test: Mapping paging-fie backed sections (large pages)\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    for (i = 0; i < sizeof(SectionProtect) / sizeof(DWORD); ++i) {
      SectionHandle = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, SectionProtect[i] | SEC_LARGE_PAGES | SEC_COMMIT, 0, (DWORD)GetLargePageMinimum(), NULL);
      if (SectionHandle != NULL) {
        BaseAddress = MapViewOfFile(SectionHandle, FileMapRights[i], 0, 0, 0);
        if (BaseAddress != NULL) {
          if (CreatePagingFileSectionTestAction(TestSet, SectionHandle, BaseAddress)) {
            printf("Paging-file backed section mapped with %s protection at address 0x%p\n", SectionProtectStr[i], BaseAddress);
          } else {
            printf("ERROR: Failed to create test action record\n");
            UnmapViewOfFile(BaseAddress);
            CloseHandle(SectionHandle);
          }
        } else {
          printf("ERROR: Failed to map the section: %u\n", GetLastError());
          CloseHandle(SectionHandle);
        }
      } else printf("ERROR: Failed to create paging file backed section with %s protection: %u\n", SectionProtectStr[i], GetLastError());
    }

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Test initialization failed: %u\n", GetLastError());

  return;
}


