
#include <stdio.h>
#include "tests.h"
#include "physmem.h"
#include "tests-physical.h"





VOID PhysicalMemoryTest(VOID)
{
  PVOID BaseAddress = NULL;
  SIZE_T MemorySize = 16384;
  LARGE_INTEGER Offset = {0};
  PTEST_SET TestSet = NULL;

  printf("Physical memory test: Mapping \\Device\\PhysicalMemory section\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    BaseAddress = PhysMemMapMemory(&MemorySize, &Offset);
    if (BaseAddress != NULL) {
	  if (CreateDevicePhysicalMemoryTestAction(TestSet, BaseAddress))
        printf("Memory mapped at address 0x%p\n", BaseAddress);
	  else printf("ERROR: Failed to create test action record\n");
	} else printf("ERROR: Failed to map physical memory: %u\n", GetLastError());

	getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Failed to create test set\n");

  return;
}


VOID SingleAWEReserveTest(VOID)
{
  PVOID BaseAddress = NULL;
  PTEST_SET TestSet = NULL;

  printf("Physical memory test: Reserving region for user physical pages (AWE extension)\n");

  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    BaseAddress = VirtualAlloc(NULL, 8192, MEM_PHYSICAL | MEM_RESERVE, PAGE_READWRITE);    
    if (BaseAddress != NULL) {
      if (CreatePrivateMemoryAllocationTestAction(TestSet, BaseAddress))
        printf("AWE region reserved at 0x%p\n", BaseAddress);
      else {
        printf("ERROR: Failed to create test action record\n");
        VirtualFree(BaseAddress, 0, MEM_RELEASE);
      }
    } else printf("ERROR: Failed to reserve AWE region: %u\n", GetLastError());

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Failed to initialize test set: %u\n", GetLastError());

  return;
}


VOID SingleAWEAllocTest(VOID)
{
  ULONG_PTR PageCount = 2;
  ULONG_PTR PageArray [2];
  PVOID BaseAddress = NULL;
  PTEST_SET TestSet = NULL;

  printf("Physical memory test: Allocating region of user physical pages (AWE extension)\n");

  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    BaseAddress = VirtualAlloc(NULL, 8192, MEM_PHYSICAL | MEM_RESERVE, PAGE_READWRITE);    
    if (BaseAddress != NULL) {
      if (AllocateUserPhysicalPages(GetCurrentProcess(), &PageCount, (PULONG_PTR)&PageArray)) {
        if (MapUserPhysicalPages(BaseAddress, PageCount, (PULONG_PTR)&PageArray)) {
          if (CreateAweTestAction(TestSet, (PULONG_PTR)&PageArray, PageCount, BaseAddress))
            printf("AWE region allocated at 0x%p\n", BaseAddress);
          else {
            printf("ERROR: Failed to create test action record\n");
            MapUserPhysicalPages(BaseAddress, PageCount, NULL);
            FreeUserPhysicalPages(GetCurrentProcess(), &PageCount, (PULONG_PTR)&PageArray);
            VirtualFree(BaseAddress, 0, MEM_RELEASE);
          }
        } else {
          printf("ERROR: Failed to map physical pages: %u\n", GetLastError()); 
          FreeUserPhysicalPages(GetCurrentProcess(), &PageCount, (PULONG_PTR)&PageArray);
          VirtualFree(BaseAddress, 0, MEM_RELEASE);
        }
      } else {
        printf("ERROR: Failed to allocate user physical pages: %u\n", GetLastError());
        VirtualFree(BaseAddress, 0, MEM_RELEASE);
      }
    } else printf("ERROR: Failed to reserve AWE region: %u\n", GetLastError());

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Failed to initialize test set: %u\n", GetLastError());

  return;
}
