
#include <stdio.h>
#include <windows.h>
#include "tests.h"
#include "tests-private.h"



const DWORD VirtualAllocRegionsProtect [] = {
	PAGE_READONLY,
	PAGE_READWRITE,
	PAGE_EXECUTE,
	PAGE_EXECUTE_READ,
	PAGE_EXECUTE_READWRITE,
	PAGE_NOACCESS
};

const PCHAR VirtualAllocRegionsProtectStr [] = {
	"PAGE_READONLY",
	"PAGE_READWRITE",
	"PAGE_EXECUTE",
	"PAGE_EXECUTE_READ",
	"PAGE_EXECUTE_READWRITE",
	"PAGE_NOACCESS"
};



VOID SingleSmallAllocTest(VOID)
{
  LONG i = 0;
  PTEST_SET TestSet = NULL;
  PVOID BaseAddress = NULL;

  printf("Private memory test: Committing region of small pages\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    for (i = 0; i < sizeof(VirtualAllocRegionsProtect) / sizeof(DWORD); ++i) {
      BaseAddress = VirtualAlloc(NULL, 8192, MEM_COMMIT, VirtualAllocRegionsProtect[i]);
      if (BaseAddress != NULL) {
        if (CreatePrivateMemoryAllocationTestAction(TestSet, BaseAddress)) {
          printf("Region of memory allocated with %s protection at address 0x%p\n", VirtualAllocRegionsProtectStr[i], BaseAddress);
        } else {
          printf("ERROR: Failed to create test action record\n");
          VirtualFree(BaseAddress, 0, MEM_RELEASE);
        }
      } else printf("ERROR: Failed to allocate memory region with %s protection: %u\n", VirtualAllocRegionsProtectStr[i], GetLastError());
    }

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Test initialization failed: %u\n", GetLastError());

  return;
}


VOID SingleSmallAllocNoCacheTest(VOID)
{
  LONG i = 0;
  PTEST_SET TestSet = NULL;
  PVOID BaseAddress = NULL;

  printf("Private memory test: Committing region of small pages, non-cached memory\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    for (i = 0; i < sizeof(VirtualAllocRegionsProtect) / sizeof(DWORD); ++i) {
      if (VirtualAllocRegionsProtect[i] == PAGE_NOACCESS)
        continue;

      BaseAddress = VirtualAlloc(NULL, 8192, MEM_COMMIT, VirtualAllocRegionsProtect[i] | PAGE_NOCACHE);
      if (BaseAddress != NULL) {
        if (CreatePrivateMemoryAllocationTestAction(TestSet, BaseAddress)) {
          printf("Region of memory allocated with %s protection at address 0x%p\n", VirtualAllocRegionsProtectStr[i], BaseAddress);
        } else {
          printf("ERROR: Failed to create test action record\n");
          VirtualFree(BaseAddress, 0, MEM_RELEASE);
        }
      } else printf("ERROR: Failed to allocate memory region with %s protection: %u\n", VirtualAllocRegionsProtectStr[i], GetLastError());
    }

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Test initialization failed: %u\n", GetLastError());

  return;
}



VOID SingleSmallAllocWriteCombinedTest(VOID)
{
  LONG i = 0;
  PTEST_SET TestSet = NULL;
  PVOID BaseAddress = NULL;

  printf("Private memory test: Committing region of small pages, write-combined memory\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    for (i = 0; i < sizeof(VirtualAllocRegionsProtect) / sizeof(DWORD); ++i) {
      if (VirtualAllocRegionsProtect[i] == PAGE_NOACCESS)
        continue;

      BaseAddress = VirtualAlloc(NULL, 8192, MEM_COMMIT, VirtualAllocRegionsProtect[i] | PAGE_WRITECOMBINE);
      if (BaseAddress != NULL) {
        if (CreatePrivateMemoryAllocationTestAction(TestSet, BaseAddress)) {
          printf("Region of memory allocated with %s protection at address 0x%p\n", VirtualAllocRegionsProtectStr[i], BaseAddress);
        } else {
          printf("ERROR: Failed to create test action record\n");
          VirtualFree(BaseAddress, 0, MEM_RELEASE);
        }
      } else printf("ERROR: Failed to allocate memory region with %s protection: %u\n", VirtualAllocRegionsProtectStr[i], GetLastError());
    }

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Test initialization failed: %u\n", GetLastError());

  return;
}


VOID SingleSmallAllocGuardTest(VOID)
{
  LONG i = 0;
  PTEST_SET TestSet = NULL;
  PVOID BaseAddress = NULL;

  printf("Private memory test: Committing region of small pages, guarded memory\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    for (i = 0; i < sizeof(VirtualAllocRegionsProtect) / sizeof(DWORD); ++i) {
      if (VirtualAllocRegionsProtect[i] == PAGE_NOACCESS)
        continue;

      BaseAddress = VirtualAlloc(NULL, 8192, MEM_COMMIT, VirtualAllocRegionsProtect[i] | PAGE_GUARD);
      if (BaseAddress != NULL) {
        if (CreatePrivateMemoryAllocationTestAction(TestSet, BaseAddress)) {
          printf("Region of memory allocated with %s protection at address 0x%p\n", VirtualAllocRegionsProtectStr[i], BaseAddress);
        } else {
          printf("ERROR: Failed to create test action record\n");
          VirtualFree(BaseAddress, 0, MEM_RELEASE);
        }
      } else printf("ERROR: Failed to allocate memory region with %s protection: %u\n", VirtualAllocRegionsProtectStr[i], GetLastError());
    }

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Test initialization failed: %u\n", GetLastError());

  return;
}


VOID SingleLargeAllocTest(VOID)
{
  LONG i = 0;
  PTEST_SET TestSet = NULL;
  PVOID BaseAddress = NULL;

  printf("Private memory test: Committing region of large pages\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    for (i = 0; i < sizeof(VirtualAllocRegionsProtect) / sizeof(DWORD); ++i) {
      BaseAddress = VirtualAlloc(NULL, GetLargePageMinimum(), MEM_COMMIT | MEM_LARGE_PAGES, VirtualAllocRegionsProtect[i]);
      if (BaseAddress != NULL) {
        if (CreatePrivateMemoryAllocationTestAction(TestSet, BaseAddress)) {
          printf("Region of memory allocated with %s protection at address 0x%p\n", VirtualAllocRegionsProtectStr[i], BaseAddress);
        } else {
          printf("ERROR: Failed to create test action record\n");
          VirtualFree(BaseAddress, 0, MEM_RELEASE);
        }
      } else printf("ERROR: Failed to allocate memory region with %s protection: %u\n", VirtualAllocRegionsProtectStr[i], GetLastError());
    }

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Test initialization failed: %u\n", GetLastError());

  return;
}


VOID SingleReserveTest(VOID)
{
  PTEST_SET TestSet = NULL;
  PVOID BaseAddress = NULL;

  printf("Private memory test: Reserving region of small pages\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    BaseAddress = VirtualAlloc(NULL, 8192, MEM_RESERVE, PAGE_READWRITE);
    if (BaseAddress != NULL) {
      if (CreatePrivateMemoryAllocationTestAction(TestSet, BaseAddress)) {
        printf("Region of memory reserved at address 0x%p\n", BaseAddress);
      } else {
        printf("ERROR: Failed to create test action record\n");
        VirtualFree(BaseAddress, 0, MEM_RELEASE);
      }
    } else printf("ERROR: Failed to reserve memory region: %u\n", GetLastError());

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Test initialization failed: %u\n", GetLastError());

	return;
}


VOID SingleProtectChangeTest(VOID)
{
  DWORD OldProtect = 0;
  PTEST_SET TestSet = NULL;
  PVOID BaseAddress = NULL;

  printf("Private memory test: Allocating memory and changing its protection\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    BaseAddress = VirtualAlloc(NULL, 8192, MEM_COMMIT, PAGE_READONLY);
    if (BaseAddress != NULL) {
      if (VirtualProtect(BaseAddress, 8192, PAGE_READWRITE, &OldProtect)) {
        if (CreatePrivateMemoryAllocationTestAction(TestSet, BaseAddress)) {
          printf("Region of memory allocated and protected at address 0x%p\n", BaseAddress);
        } else {
          printf("ERROR: Failed to create test action record\n");
          VirtualFree(BaseAddress, 0, MEM_RELEASE);
        }
	  } else {
        printf("ERROR: Failed to set memory protection at 0x%p, %u\n", BaseAddress, GetLastError());
        VirtualFree(BaseAddress, 0, MEM_RELEASE);
	  }
    } else printf("ERROR: Failed to allocate memory region: %u\n", GetLastError());

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Test initialization failed: %u\n", GetLastError());

  return;
}


VOID SingleWriteWatchTest(VOID)
{
  PTEST_SET TestSet = NULL;
  PVOID BaseAddress = NULL;

  printf("Private memory test: Creating write-watch region of small pages\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    BaseAddress = VirtualAlloc(NULL, 8192, MEM_RESERVE | MEM_WRITE_WATCH, PAGE_READWRITE);
    if (BaseAddress != NULL) {
      if (CreatePrivateMemoryAllocationTestAction(TestSet, BaseAddress)) {
        printf("Write-watch region of memory created at address 0x%p\n", BaseAddress);
      } else {
        printf("ERROR: Failed to create test action record\n");
        VirtualFree(BaseAddress, 0, MEM_RELEASE);
      }
    } else printf("ERROR: Failed to create write-watch memory region: %u\n", GetLastError());

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Test initialization failed: %u\n", GetLastError());

  return;
}


VOID TwoPhaseSmallAllocTest(VOID)
{
  LONG i = 0;
  PTEST_SET TestSet = NULL;
  PVOID BaseAddress = NULL;

  printf("Private memory test: Reserving and then committing region of small pages\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    for (i = 0; i < sizeof(VirtualAllocRegionsProtect) / sizeof(DWORD); ++i) {
      BaseAddress = VirtualAlloc(NULL, 16384, MEM_RESERVE, VirtualAllocRegionsProtect[i]);
      if (BaseAddress != NULL) {
        BaseAddress = VirtualAlloc(BaseAddress, 16384, MEM_COMMIT, VirtualAllocRegionsProtect[i]);
        if (BaseAddress != NULL) {
          if (CreatePrivateMemoryAllocationTestAction(TestSet, BaseAddress)) {
            printf("Region of memory allocated with %s protection at address 0x%p\n", VirtualAllocRegionsProtectStr[i], BaseAddress);
          } else {
            printf("ERROR: Failed to create test action record\n");
            VirtualFree(BaseAddress, 0, MEM_RELEASE);
          }
        } else {
          printf("ERROR: Failed to allocate memory region with %s protection at address 0x%p: %u\n", VirtualAllocRegionsProtectStr[i], BaseAddress, GetLastError());
          VirtualFree(BaseAddress, 0, MEM_RELEASE);
        }
      } else printf("ERROR: Failed to reserve memory region with %s protection: %u\n", VirtualAllocRegionsProtectStr[i], GetLastError());
    }

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Test initialization failed: %u\n", GetLastError());

  return;
}


