
#include <stdio.h>
#include <windows.h>
#include "tests.h"
#include "secmem.h"
#include "tests-secured.h"



/** Pouzivana opravneni stranek pri alokaci a zabezpeceni regionu. */
static const DWORD TestProtection[] = {
  PAGE_READONLY,
  PAGE_READWRITE
};

/** Nazvy opravneni stranek pouzivanych pri alokaci a zabezpeceni regionu. */
static const char *TestProtectionStr[] = {
  "PAGE_READONLY",
  "PAGE_READWRITE"
};



VOID WholeSecureTest(VOID)
{
  LONG i = 0;
  PTEST_SET TestSet = NULL;
  HANDLE Handle = NULL;
  PVOID BaseAddress = NULL;

  printf("Secured memory test: securing the whole allocated region of memory\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    for (i = 0; i < sizeof(TestProtection) / sizeof(DWORD); ++i) {
      BaseAddress = VirtualAlloc(NULL, 8192, MEM_COMMIT, TestProtection[i]);
      if (BaseAddress != NULL) {
        if (CreatePrivateMemoryAllocationTestAction(TestSet, BaseAddress)) {
          Handle = SecMemSecureMemory(BaseAddress, 8192, TestProtection[i]);
          if (Handle != NULL) {
            if (CreateSecuredRegionTestAction(TestSet, Handle))
              printf("Secured region with %s protection at address 0x%p\n", TestProtectionStr[i], BaseAddress);
            else {
              printf("ERROR: Failed to create test action record\n");
              SecMemUnsecureMemory(Handle);
            }
          } else printf("ERROR: Failed to secure memory at 0x%p\n", BaseAddress);
		} else printf("ERROR: Failed to create test action for memory allocation\n");
      } else printf("ERROR: Failed to allocate memory with %s protection\n", TestProtectionStr[i]);
	}

	getchar();
	TestSetFree(TestSet);
  }

  return;
}


/** Test #2
 */
VOID WholeSecureTest2(VOID)
{
  HANDLE Handle = NULL;
  PVOID Address = NULL;
  PTEST_SET TestSet = NULL;

  printf("Secured memory test: securing the whole allocated region of memory (read/write) for read-only access\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    Address = VirtualAlloc(NULL, 8192, MEM_COMMIT, PAGE_READWRITE);
    if (Address != NULL) {
      if (CreatePrivateMemoryAllocationTestAction(TestSet, Address)) {
        Handle = SecMemSecureMemory(Address, 8192, PAGE_READONLY);
        if (Handle != NULL) {
          if (CreateSecuredRegionTestAction(TestSet, Handle))
            printf("Secured region at address 0x%p\n", Address);
          else {
            printf("ERROR: Failed to create test action record\n");
            SecMemUnsecureMemory(Handle);
          }
        } else printf("ERROR: Failed to secure memory at 0x%p\n", Address);
      } else printf("ERROR: Failed to create test action record for memory allocation\n");
    } else printf("ERROR: Failed to allocate memory: %u\n", GetLastError());

    getchar();
    TestSetFree(TestSet);
  } else printf("ERROR: Failed to create test set\n");


  return;
}


/** Test #3
 */
VOID PartialSecureTest(VOID)
{
  LONG i = 0;
  HANDLE Handle = NULL;
  PVOID BaseAddress = NULL;
  PTEST_SET TestSet = NULL;

  printf("Secured memory test: securing the part of the allocated region of memory\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    for (i = 0; i < sizeof(TestProtection) / sizeof(DWORD); ++i) {
      BaseAddress = VirtualAlloc(NULL, 16384, MEM_COMMIT, TestProtection[i]);
      if (BaseAddress != NULL) {
        if (CreatePrivateMemoryAllocationTestAction(TestSet, BaseAddress)) {
          Handle = SecMemSecureMemory((PCHAR)BaseAddress + 4096, 4096, TestProtection[i]);
          if (Handle != NULL) {
            if (CreateSecuredRegionTestAction(TestSet, Handle))
              printf("Secured region with %s protection at address 0x%p\n", TestProtectionStr[i], BaseAddress);
            else {
              printf("ERROR: Failed to create test action record\n");
              SecMemUnsecureMemory(Handle);
            }
          } else printf("ERROR: Failed to secure memory at 0x%p\n", BaseAddress);
        } else printf("ERROR: Failed to create test action record for memory allocation\n");
      } else printf("ERROR: Failed to allocate memory: %u\n", GetLastError());
    }

	getchar();
	TestSetFree(TestSet);
  }

  return;
}


/** Test #4
 */
VOID MultipleSecureTest(VOID)
{
  LONG i, j = 0;
  HANDLE Handle = NULL;
  PVOID BaseAddress = NULL;
  PTEST_SET TestSet = NULL;

  printf("Secured memory test: securing more distinct parts of the allocated region of memory\n");
  TestSet = TestSetCreate();
  if (TestSet != NULL) {
    for (i = 0; i < sizeof(TestProtection) / sizeof(DWORD); ++i) {
      BaseAddress = VirtualAlloc(NULL, 6*4096, MEM_COMMIT, TestProtection[i]);
      if (BaseAddress != NULL) {
        if (CreatePrivateMemoryAllocationTestAction(TestSet, BaseAddress)) {
          for (j = 0; j < 3; ++j) {
            Handle = SecMemSecureMemory((PCHAR)BaseAddress + 4096*(1 + 2*j), 4096, TestProtection[i]);
            if (Handle != NULL) {
              if (CreateSecuredRegionTestAction(TestSet, Handle))
                printf("Secured part of region with %s protection at address 0x%p\n", TestProtectionStr[i], BaseAddress);
			  else {
                printf("ERROR: Failed to create action record for memory securing operation\n");
                SecMemUnsecureMemory(Handle);
              }
            } else printf("ERROR: Failed to secure memory at 0x%p\n", (PCHAR)BaseAddress + 4096*(1 + 2*j));
          }
        } else {
          printf("ERROR: Failed to create action record for memory allocation\n");
          VirtualFree(BaseAddress, 0, MEM_RELEASE);
        }
      } else printf("ERROR: Failed to allocate memory: %u\n", GetLastError());
    }

	getchar();
	TestSetFree(TestSet);
  }

  return;
}
